#! /usr/bin/env python
# -*- coding: utf-8 -*-
__all__ = [
    'T',
    'ArrayLike',
    'PandasLike',
    'DiosLikeT',
    'FuncReturnT',
    'FreqString',
    'ColumnName',
    'IntegerWindow',
    'TimestampColumnName',
    'CurveFitter',
    "ExternalFlag",
    "CallStack",
    "CalledStack"
]

from typing import TypeVar, Union, NewType, List, Tuple, Optional
from typing_extensions import Protocol, Literal
import numpy as np
import pandas as pd
from dios import DictOfSeries
from saqc.core.flags import Flags
from saqc.core.lib import SaQCFunction, ColumnSelector, APIController


T = TypeVar("T")
ArrayLike = TypeVar("ArrayLike", np.ndarray, pd.Series, pd.DataFrame)
PandasLike = Union[pd.Series, pd.DataFrame, DictOfSeries]
DiosLikeT = Union[DictOfSeries, pd.DataFrame]

FuncReturnT = Tuple[DictOfSeries, Flags]

ExternalFlag = Union[str, float, int]

# we only support fixed length offsets
FreqString = NewType("FreqString", Literal["D", "H", "T", "min", "S", "L", "ms", "U", "us", "N"])

CallStack = List[Tuple[ColumnSelector, APIController, SaQCFunction]]
CalledStack = List[Tuple[ColumnSelector, Optional[SaQCFunction]]]

# we define a bunch of type aliases, mostly needed to generate appropiate fuzzy data through hypothesis
ColumnName = NewType("ColumnName", str)
IntegerWindow = NewType("IntegerWindow", int)
TimestampColumnName = TypeVar("TimestampColumnName", bound=str)

# needed for deeper typy hinting magic
class CurveFitter(Protocol):
    def __call__(self, data: np.ndarray, *params: float) -> np.ndarray:
        ...
